var data;

var path = d3.geo.path();

var w = 960,
	h = 500,
	scale = 1.0;

var plotWidth = 500;
var plotHeight = 500;
var plotMargins = {
	"top": 20,
	"bottom": 80,
	"left": 80,
	"right": 20,
}

var brewSeq = [
	[255, 255, 217],
	[237, 248, 217],
	[199, 233, 180],
	[127, 205, 187],
	[65, 182, 196],
	[29, 145, 192],
	[34, 94, 168],
	[37, 52, 148],
	[8, 29, 88],
	]
var brewDiverge = [
	[84, 48, 5],
	[140, 81, 10],
	[191, 129, 45],
	[223, 194, 125],
	[246, 232, 195],
	[245, 245, 245],
	[199, 234, 229],
	[128, 205, 193],
	[53, 151, 143],
	[1, 102, 94],
	[0, 60, 48],
]
var brewDiverge10 = [
	[64, 0, 75],
	[118, 42, 131],
	[153, 112, 171],
	[194, 165, 207],
	[231, 212, 232],
	[217, 240, 211],
	[166, 219, 160],
	[90, 174, 97],
	[27, 120, 55],
	[0, 68, 27],
]
var brewBi = [
	[27,158,119],
	[117,112,179],
]
var neutralColor = d3.rgb(111,111,111);
var stateId = {};
var csvData = [];
var csvAttributes = {};
var jsonData = [];

function getColors() {
	if (!csvAttributes[cKey]) return brewDiverge;
	if (csvAttributes[cKey].format == "pct") return brewDiverge;
	if (csvAttributes[cKey].format == "pctchg") return brewDiverge10;
	if (csvAttributes[cKey].format == "bi") return brewBi;
	if (csvAttributes[cKey].format == "one") return brewDiverge10;
	if (csvAttributes[cKey].format == "abs") return brewDiverge10;
	if (csvAttributes[cKey].format == "quant") return brewSeq;
	return brewBi;
}
function getColor(i) {
	var colors = getColors();
	var c = i < colors.length ? colors[i] : neutralColor;
	return d3.rgb(c[0], c[1], c[2]);
}

var plotInnerWidth = plotWidth - plotMargins.left - plotMargins.right;
var plotInnerHeight = plotHeight - plotMargins.top - plotMargins.bottom;

var xKey = "pacfinance";
var yKey = "firstpredict";
var cKey = "firstbailoutvote";
var xScale = d3.scale.linear().domain([0,100]).range([0,plotInnerWidth]);
var yScale = d3.scale.linear().domain([0,100]).range([plotInnerHeight,0]);
var cScale = d3.scale.linear().domain([0,100]).rangeRound([0,10]);
var xFormat = "pct";
var yFormat = "pct";
var cFormat = "pct";

function updateScale(scale, key) {
	if (csvAttributes[key].format == "pct") {
		scale.domain([0,100]);
		return "pct";
	} if (csvAttributes[key].format == "pctchg") {
		var vals = csvData.map(function(d) { return Math.abs(d[key]); });
		scale.domain([-1.07 * Math.max.apply(Math, vals), 1.07 * Math.max.apply(Math, vals)]);
		return "pct";
	} else if (csvAttributes[key].format == "quant") {
		var vals = csvData.map(function(d) { return d[key]; });
		scale.domain([.93 * Math.min.apply(Math, vals), 1.07 * Math.max.apply(Math, vals)]);
		return "quant";
	} else if (csvAttributes[key].format == "abs") {
		scale.domain([-1,1]);
		return "pcte";
	} else if (csvAttributes[key].format == "one") {
		scale.domain([0,1]);
		return "pcte";
	} else if (csvAttributes[key].format == "bi") {
		scale.domain([0,1]);
		return "pcte";
	} else {
		return "";
	}
}

function setXKey(key) {
	xKey = key;
	xFormat = updateScale(xScale, key);
}
function setYKey(key) {
	yKey = key;
	yFormat = updateScale(yScale, key);
}
function setCKey(key) {
	cKey = key;
	cFormat = updateScale(cScale, key);
	cScale.rangeRound([0, getColors().length - 1]);
}

var brushing = false;
function brushstart() {
	onMouseOut(hover.key, hover.district, hover.point);
	brushing = true;
	plot.selectAll("path.dot")
		.transition()
			.setupPlotAttrs()
			.delay(0)
			.duration(500)
	districts.selectAll("path")
		.transition()
			.setupMapAttrs()
			.delay(0)
			.duration(500)
}
function brush() {
	plot.selectAll("path.dot")
		.setupPlotAttrs()
	districts.selectAll("path")
		.setupMapAttrs()

	plotBrushLines.selectAll("line").remove();
	if (!brush.empty()) {
		var e = brush.extent();
		plotBrushLines.append("line")
			.attr("x1", xScale.range()[0] - 10)
			.attr("y1", yScale(e[0][1]))
			.attr("x2", xScale(e[0][0]))
			.attr("y2", yScale(e[0][1]))
			.attr("stroke", "#444")
			.attr("stroke-width", "1px")
		plotBrushLines.append("line")
			.attr("x1", xScale.range()[0] - 10)
			.attr("y1", yScale(e[1][1]))
			.attr("x2", xScale(e[0][0]))
			.attr("y2", yScale(e[1][1]))
			.attr("stroke", "#444")
			.attr("stroke-width", "1px")
		plotBrushLines.append("line")
			.attr("x1", xScale(e[0][0]))
			.attr("y1", yScale.range()[0] + 10)
			.attr("x2", xScale(e[0][0]))
			.attr("y2", yScale(e[0][1]))
			.attr("stroke", "#444")
			.attr("stroke-width", "1px")
		plotBrushLines.append("line")
			.attr("x1", xScale(e[1][0]))
			.attr("y1", yScale.range()[0] + 10)
			.attr("x2", xScale(e[1][0]))
			.attr("y2", yScale(e[1][1]))
			.attr("stroke", "#444")
			.attr("stroke-width", "1px")
	}
}
function brushend() {
	brushing = false;
	plot.selectAll("path.dot")
		.transition()
			.setupPlotAttrs()
			.delay(0)
			.duration(500)
	districts.selectAll("path")
		.transition()
			.setupMapAttrs()
			.delay(0)
			.duration(500)
}

var brush = d3.svg.brush()
	.on("brushstart", brushstart)
	.on("brush", brush)
	.on("brushend", brushend)

var plot =
	d3.select("#plot")
	.append("svg:svg")
		.attr("width", plotWidth)
		.attr("height", plotHeight)
		.attr("pointer-events", "all")

plot = plot.append("svg:g")
	.attr("transform", "translate(" + plotMargins.left + "," + plotMargins.top + ")")
	.attr("id", "innerPlot")

plot.append("svg:rect")
	.attr("width", plotInnerWidth)
	.attr("height", plotInnerHeight)
	.attr("fill", "#aaa")

function points() {
	return plot.selectAll("path.dot")
}

var plotBrush = plot.append("g")
	.attr("class", "brush")

var plotBrushLines = plot

var plotRules = plot.append("svg:g").classed("rules", true);

function make_x_axis() {
	return d3.svg.axis()
		.scale(xScale)
		.orient("bottom")
		.ticks(10)
		.tickSize(10,5,2)
}
function make_y_axis() {
	return d3.svg.axis()
		.scale(yScale)
		.orient("left")
		.ticks(10)
		.tickSize(10,5,2)
}
function make_c_axis() {
	return d3.svg.axis()
		.scale(cScale.copy().range([0, 150]))
		.orient("bottom")
		.ticks(2.5)
		.tickSize(0)
}

function changeEncoding() {
	refreshMap(function(s) { return s.transition().delay(50).duration(1000) });
	refreshPlot(function(s) { return s.transition().delay(50).duration(1000) });
}

var dragging = false;
function dragstart() {
	dragging = true;
}
function dragend() {
	dragging = false;
}

var mapWidth = 960
var mapHeight = 500
var outerMap = 
	d3.select("#map")
	.append("svg:svg")
		.attr("width", w)
		.attr("height", h)


map = outerMap.append("svg:g")
		.call(d3.behavior.zoom().on("zoom", redraw).scaleExtent([0.8, 30]))
		.on("mousedown", dragstart)
		.on("mouseup", dragend)

var innerMap = map.append("svg:g")

var districts = innerMap
	.append("svg:g")
		.attr("id", "districts")
		.attr("pointer-events", "all")

var states = innerMap
	.append("svg:g")
		.attr("id", "states")

var districtTop = innerMap
	.append("svg:g")
		.attr("id", "districtTop")
		.attr("pointer-events", "all")

var mapRulesPos = [0, 0]
var mapRulesScale = [1, 1]
var mapRules = map
	.append("svg:g")
	.classed("rules", true)
	.attr("transform", "translate(520,450)")

mapRules.selectAll("rect")
		.data(Array(11))
	.enter()
		.append("rect")
		.attr("y", -16)
		.attr("height", 16)
		.attr("stroke", "#000")
		.attr("stroke-width", "1px")


function refreshMap(transition) {
	transition = transition || function (s) { return s; }
	transition(districts.selectAll("path"))
		.setupMapAttrs()
	transition(districtTop.selectAll("path"))
		.setupMapAttrs()
	transition(states.selectAll("path"))
		.attr("stroke-width", (1.5 / scale) + "px")

	var nColors = getColors().length;
	mapRules.selectAll("g").remove();
	mapRules.selectAll("text").remove();

	transition(mapRules.selectAll("rect"))
		.attr("x", function(d,i) { return 170 * (i / nColors) - 10 })
		.attr("width", 170 / nColors)
		.attr("stroke-opacity", function(d,i) { return i < nColors ? 1 : 0 })
		.attr("fill-opacity", function(d,i) { return i < nColors ? 1 : 0 })
		.attr("fill", function(d,i) { return getColor(i).toString() })

	mapRules.append("svg:g").classed("grid x_grid", true)
			.call(make_c_axis()
				)
	mapRules.append("svg:text").classed("text label", true)
		.text(csvAttributes[cKey].description)
		.attr("transform", "translate(20,40)")
}

function refreshPlot(transition) {
	transition = transition || function (s) { return s; }
	plotRules.selectAll("g").remove();
	plotRules.selectAll("text").remove();
	plotRules.append("svg:g").classed("grid x_grid", true)
			.attr("transform", "translate(0, " + plotInnerHeight + ")")
			.call(make_x_axis()
				)
	plotRules.append("svg:g").classed("grid y_grid", true)
			.call(make_y_axis()
				)

	plotRules.append("svg:text").classed("text label", true)
		.text(csvAttributes[xKey].description)
		.attr("transform", "translate(40," + (plotInnerHeight + 40) + ")")

	plotRules.append("svg:text").classed("text label", true)
		.text(csvAttributes[yKey].description)
		.attr("transform", "translate(-40," + (plotInnerHeight - 100) + ") rotate(-90)")

	plot.call(brush.x(xScale).y(yScale))
	transition(points()).setupPlotAttrs()
}


function redraw() {
	var t = d3.event.translate;
	var s = d3.event.scale;
	function clamp(v,l,h) { return v < l ? l : v > h ? h : v; }
	t[0] = clamp(t[0], (-1072 * s) + 960, 100);
	t[1] = clamp(t[1], (-575 * s) + 500, 50);

	innerMap.attr("transform",
		"translate(" + t + ")"
		+ " scale(" + s + ")");
	districtTop.selectAll("path")
		.attr("stroke-width", mapStrokeWidth);
	if (scale != s) {
		scale = s;
		refreshMap();
	}
}

var districtMap = {};

function showDistrictInfo(id) {
	var data = districtMap[id].data;
	var ex = districtMap[id].ex;
	var prop = districtMap[id].properties;
	if (data && prop && ex) {
		document.getElementById("infoCard_state").innerHTML = "State: " + data["state"];
		document.getElementById("infoCard_district").innerHTML = prop["NAMELSAD"];
		document.getElementById("infoCard_rep").innerHTML = "Representative: " + data["nameoftherepresentative"];
		document.getElementById("infoCard_party").innerHTML = "Party: " + (data["party"] == "R" ? "Republican" : "Democrat");
		document.getElementById("infoCard_1stpredict").innerHTML = csvAttributes["1stPredict"].description + ": " + data["1stPredict"];
		document.getElementById("infoCard_1stvote").innerHTML = csvAttributes["1stBailoutVote"].description + ": " + data["1stBailoutVote"];
		document.getElementById("infoCard_2ndvote").innerHTML = csvAttributes["2ndBailoutVote"].description + ": " + data["2ndBailoutVote"];
		document.getElementById("infoCard_xvar").innerHTML = csvAttributes[xKey].description + ": " + data[xKey];
		document.getElementById("infoCard_yvar").innerHTML = csvAttributes[yKey].description + ": " + data[yKey];
		document.getElementById("infoCard_cvar").innerHTML = csvAttributes[cKey].description + ": " + data[cKey];
	}
}


function getPointByKey(key) {
	return document.getElementById("point_" + key);
}
function getDistrictByKey(key) {
	return document.getElementById("district_" + key);
}

var mapFill = function(data) {
	var key = data.key;
	var d = districtMap[key].data;
	if (!d || !d[cKey] || d[cKey] == "") return neutralColor;
	if (!brush.empty()) {
		var e = brush.extent();
		if (e && !(e[0][0] <= d[xKey] && d[xKey] <= e[1][0]
				&& e[0][1] <= d[yKey] && d[yKey] <= e[1][1]))
			return neutralColor;
	}
	var b = getColors()[cScale(d[cKey])];
	if (!b) b = neutralColor;
	return d3.rgb(b[0], b[1], b[2]);
}
function mapStroke(data) {
	if (data.hover) {
		var f = mapFill(data);
		var h = f.hsl();
		h.h = h.h > 180 ? h.h - 180 : h.h + 180;
		h.s = 1;
		h.l = 1.0 - h.l;
		return h.rgb();
	}
	else return d3.rgb(127,127,127);
}
function mapStrokeWidth(data) {
	return ((data.hover ? 4 : 0.5) / scale) + "px";
}
var plotFill = function(data) {
	return mapFill(data);
}
function plotStroke(data) {
	return mapStroke(data);
}
function plotStrokeWidth(data) {
	return (data.hover ? 4 : 1) + "px";
}

var hover = {};
function onMouseOver(key, district, point) {
	if (dragging || !point || !district) return;

	hover = { "key": key, "district": district, "point": point }
	showDistrictInfo(key);

	district.__data__.hover = true;
	point.__data__.hover = true;

	district.parentNode.removeChild(district);
	document.getElementById("districtTop").appendChild(district);
	d3.select(district).setupMapAttrs();

	point.parentNode.removeChild(point);
	document.getElementById("innerPlot").appendChild(point);
	d3.select(point).setupPlotAttrs();
}

function onMouseOut() {
	var key = hover.key, point = hover.point, district = hover.district;
	if (!point || !district) return;

	district.__data__.hover = false;
	point.__data__.hover = false;

	district.parentNode.removeChild(district);
	document.getElementById("districts").appendChild(district);
	d3.select(district).setupMapAttrs();
	d3.select(point).setupPlotAttrs();
}

d3.selection.prototype.setupMapAttrs = 
	d3.selection.enter.prototype.setupMapAttrs =
	d3.transition.prototype.setupMapAttrs =
	function() {
		return this
			.attr("fill", function(d) { return mapFill(d).toString(); })
			.attr("stroke", function(d) { return mapStroke(d).toString(); })
			.attr("stroke-width", mapStrokeWidth)
			.attr("stroke-opacity", "1")
	}

d3.selection.prototype.setupPlotAttrs = 
	d3.selection.enter.prototype.setupPlotAttrs =
	d3.transition.prototype.setupPlotAttrs = function() {
		return this
			.attr("fill", function(d) { return plotFill(d).toString(); })
			.attr("stroke", function(d) { return plotStroke(d).toString(); })
			.attr("stroke-width", plotStrokeWidth)
			.attr("transform", plotPosition)
			.attr("fill-opacity", function(d) { return d[xKey] && d[yKey] ? 1 : 0; })
			.attr("stroke-opacity", function(d) { return d[xKey] && d[yKey] ? 1 : 0; })
}

function districtIdFromGeo(data) {
	return data.properties["STATEFP"] + "." + data.properties["CD112FP"]
}

function districtIdFromData(data) {
	var cd = data["district"] == "At Large" ? "00" : data["cd"];
	return stateId[data["state"]] + "." + cd;
}

function importGeoJSON(json) {
	jsonData = json.features;
	var features = json.features;
	for (var i = 0; i < features.length; i++) {
		var key = districtIdFromGeo(features[i]);
		features[i].key = key;
		if (!districtMap[key]) districtMap[key] = {};
		districtMap[key].properties = features[i].properties;
	}
}

d3.json("2011_us_cd_simple.json", function(json) {
		importGeoJSON(json)
		setupData()
		})

var plotPosition = function(d) {
	var ret = "translate(" + xScale(d[xKey]) + "," + yScale(d[yKey]) + ")";
	return ret;
}

d3.json("us-states.json", function(json) {
		states.selectAll("path")
			.data(json.features)
		.enter()
			.append("path")
			.attr("d", path)
		})

var controls_radio_heading = d3.select("#controls_radio_heading")
var radio_x = d3.select("#controls_xaxis")
var radio_y = d3.select("#controls_yaxis")
var radio_c = d3.select("#controls_color")

/*
function formatCsvData() {
	if (csvData && csvAttributes) {
		csvData.forEach(function(d) {
			for (var v in d)
				if (csvAttributes[v] && csvAttributes[v].format == "quant")
					d[v] = d[v].replace(/,/g, "");
					})
	}
}
*/

function importDataSet() {
	csvData.forEach(function(d) {
		var key = districtIdFromData(d);
		d.key = key;
		if (!districtMap[key]) districtMap[key] = {};
		districtMap[key].data = d;
	});
}

function setupRadios() {
	var encodable = [null];
	for (var s in csvAttributes) {
		var d = csvAttributes[s];
		if (d.encodable > 0) encodable[encodable.length] = d;
	}
	controls_radio_heading.selectAll("td")
		.data(encodable)
	.enter()
		.append("td")
		.html(function(d) { return d.key })

	function addRadio(d, name) {
		return d.selectAll("td")
			.data(encodable)
		.enter()
			.append("td")
			.append("input")
			.attr("type", "radio")
			.attr("name", name)
	}

	addRadio(radio_x, "radio_x")
		.attr("checked", function(d) { return d.key == xKey ? "checked" : null })
		.on("click", function(d) { setXKey(d.key); changeEncoding(); })
	addRadio(radio_y, "radio_y")
		.attr("checked", function(d) { return d.key == yKey ? "checked" : null })
		.on("click", function(d) { setYKey(d.key); changeEncoding(); })
	addRadio(radio_c, "radio_c")
		.attr("checked", function(d) { return d.key == cKey ? "checked" : null })
		.on("click", function(d) { setCKey(d.key); changeEncoding(); })

}

function initMap() {
	districts.selectAll("path")
		.data(jsonData, function(d) { return d.key; })
	.enter()
		.append("path")
		.setupMapAttrs()
		.attr("d", path)
		.attr("id", function(d) { return "district_" + d.key; })
		.on("mouseover",
			function(d) {
				onMouseOver(d.key, this, getPointByKey(d.key));
			})
		.on("mouseout",
			function(d) {
				onMouseOut(d.key, this, getPointByKey(d.key));
			})
}

function initPlot() {
	points()
		.data(csvData)
	.enter()
		.append("svg:path")
		.attr("id", function(d) { return "point_" + d.key; })
		.attr("x", 0)
		.attr("y", 0)
		.attr("class", "dot")
		.attr("d", d3.svg.symbol())
		.setupPlotAttrs()
		.on("mouseover", function(d) {
				if (!brushing) {
					onMouseOver(d.key, getDistrictByKey(d.key), this);
				}
			})
		.on("mouseout", function(d) {
				if (!brushing) {
					onMouseOut(d.key, getDistrictByKey(d.key), this);
				}
			})
}

function setupData() {
	if (jsonData && stateId && csvData && csvAttributes) {
		importDataSet();
		initPlot();
		initMap();
		setXKey("pacFinance");
		setYKey("1stPredict");
		setCKey("1stBailoutVote");
		setupRadios();
		refreshMap();
		refreshPlot();
	}
}
d3.csv("cd111_112_0809rt.csv", function(csv) {
	csv.forEach(function(d) {
		stateId[d["State"]] = d["CD2"].substring(0,2);
		var key = d["CD2"].substring(0,2) + "." + d["CD2"].substring(2);
		if (!districtMap[key]) districtMap[key] = {};
		districtMap[key].ex = d;
	});
	setupData();
})

d3.csv("visproject_attributes.csv", function(csv) {
	csv.forEach(function(d) { csvAttributes[d.key] = d; })
	setupData();
})

d3.csv("visproject.csv", function(csv) {
	csv.forEach(function(d) { while (d["cd"].length < 2) d["cd"] = "0" + d["cd"]; });
	csvData = csv;
	setupData();
})
